clear variables
path2data = '/scratch/fmc1q07/altimetry_along_track/my.cmems-du.eu/Core/SEALEVEL_GLO_PHY_L3_MY_008_062/'; % directory containing original data
pathout = '/scratch/fmc1q07/pass-swio/cmems_along_track/'; % directory where processed data is saved

% select altimetry mission
alti = 's3b'; % 'jason', 's3a', 's3b'

% select domain
xmax = 56; % maximum longitude
xmin = 36; % minimum longitude
ymax = -8; % maximum latitude
ymin = -30; % minimum latitude

% select time period
yr0 = 2002; % start year
yrf = 2021; % final year

folders = dir([path2data,'cmems*']);
folders = folders([folders(:).isdir]);

fns = {folders(:).name};
if strcmp(alti,'jason')
    k = contains(fns,'_j1-') | contains(fns,'_j2-') | contains(fns,'_j3-');
elseif strcmp(alti,'s3a')
    k = contains(fns,'_s3a');
elseif strcmp(alti,'s3b')
    k = contains(fns,'_s3b');
end
folders = folders(k);
nf = length(folders);

n = 1e8; % a big number for pre-allocation purposes
clear data
lon = zeros(n,1);
lat = zeros(n,1);
t = zeros(n,1);
y = zeros(n,1);
yf = zeros(n,1);
dac = zeros(n,1);
track = zeros(n,1);
cycle = zeros(n,1);
mission = cell(n,1);
is = 1;
for yr=yr0:yrf
    disp(yr)
    for m=1:12
        for i=1:nf
            years = dir([path2data,folders(i).name]);
            years = struct2cell(years([years(:).isdir]));
            years = years(1,:)';
            if ~ismember(num2str(yr),years) % if there is no data for this year, continue
                continue
            end
            months = dir([path2data,folders(i).name,'/',num2str(yr)]);
            months = struct2cell(months([months(:).isdir]));
            months = months(1,:)';
            
            if ~ismember(num2str(m,'%02d'),months) % if there is no data for this month, continue
                continue
            end
            fn = dir([path2data,folders(i).name,'/',num2str(yr),'/',num2str(m,'%02d'),'/*.nc']);
            nf2 = length(fn);
            for j=1:nf2
                
                missioni = strsplit(fn(j).name,'_');
                missioni = cellstr(missioni{3});
                fid = netcdf.open([path2data,folders(i).name,'/',num2str(yr),'/',num2str(m,'%02d'),'/',fn(j).name],'NC_NOWRITE');
                id = netcdf.inqVarID(fid,'longitude');
                lo = netcdf.getVar(fid,id,'double');
                sf = netcdf.getAtt(fid,id,'scale_factor','double');
                lo = lo.*sf;
                
                id = netcdf.inqVarID(fid,'latitude');
                la = netcdf.getVar(fid,id,'double');
                sf = netcdf.getAtt(fid,id,'scale_factor','double');
                la = la.*sf;
                
                id = netcdf.inqVarID(fid,'time');
                ti = netcdf.getVar(fid,id,'double') + datenum(1950,1,1);
                
                id = netcdf.inqVarID(fid,'track');
                tracki = netcdf.getVar(fid,id,'double');
                
                id = netcdf.inqVarID(fid,'cycle');
                cyclei = netcdf.getVar(fid,id,'double');
                
                id = netcdf.inqVarID(fid,'sla_filtered');
                hf = netcdf.getVar(fid,id,'double');
                sf = netcdf.getAtt(fid,id,'scale_factor','double');
                fv = netcdf.getAtt(fid,id,'_FillValue','double');
                of = netcdf.getAtt(fid,id,'add_offset','double');
                hf(hf==fv) = NaN;
                hf = hf*sf + of;
                
                id = netcdf.inqVarID(fid,'dac');
                daci = netcdf.getVar(fid,id,'double');
                sf = netcdf.getAtt(fid,id,'scale_factor','double');
                fv = netcdf.getAtt(fid,id,'_FillValue','double');
                of = netcdf.getAtt(fid,id,'add_offset','double');
                daci(daci==fv) = NaN;
                daci = daci*sf + of;
                
                id = netcdf.inqVarID(fid,'sla_unfiltered');
                h = netcdf.getVar(fid,id,'double');
                sf = netcdf.getAtt(fid,id,'scale_factor','double');
                fv = netcdf.getAtt(fid,id,'_FillValue','double');
                of = netcdf.getAtt(fid,id,'add_offset','double');
                h(h==fv) = NaN;
                h = h*sf + of;
                
                netcdf.close(fid);
                
                lo(lo>180) = lo(lo>180) - 360;
                k = lo>=xmin & lo<=xmax & la>=ymin & la<=ymax & ~isnan(h);
                ie = is + sum(k) - 1;
                lon(is:ie) = lo(k);
                lat(is:ie) = la(k);
                t(is:ie) = ti(k);
                y(is:ie) = h(k);
                yf(is:ie) = hf(k);
                dac(is:ie) = daci(k);
                track(is:ie) = tracki(k);
                cycle(is:ie) = cyclei(k);
                mission(is:ie) = missioni;
                is = ie + 1;
            end
        end
    end
end

k = ~(t==0);
lon = lon(k);
lat = lat(k);
t = t(k);
y = y(k);
yf = yf(k);
dac = dac(k);
track = track(k);
cycle = cycle(k);
mission = mission(k);

mm = unique(mission);
c = containers.Map;
for ii=1:length(mm)
    c(mm{ii}) = ii;
end
pp = zeros(length(lon),1);
for ii=1:length(mm)
    k = strcmp(mission,mm{ii});
    pp(k) = c(mm{ii});
end

% cycle numbers are restarted in each Jason mission. To merge the various missions we need
% to obtain a variable that accounts for both the cycle number and the
% mission. We do that below.
for i=1:length(mm)
cycle(pp == i) = cycle(pp == i) + (i-1)*999999;
end

% organize data by pass
track_unique = unique(track);
ntrack = length(track_unique);
for i=1:ntrack
    k = find(track == track_unique(i));
    cyclei = cycle(k);
    cycle_unique = unique(cyclei);
    ncycle = length(cycle_unique);
    
    loni = ones(ncycle,length(k))*999;
    lati = ones(ncycle,length(k))*999;
    yi = ones(ncycle,length(k));
    yfi = ones(ncycle,length(k));
    daci = ones(ncycle,length(k));
    ti = zeros(ncycle,1);
    for j=1:ncycle
        ki = find(cyclei == cycle_unique(j));
        ni = length(ki);
        loni(j,1:ni) = lon(k(ki));
        lati(j,1:ni) = lat(k(ki));
        yi(j,1:ni) = y(k(ki));
        yfi(j,1:ni) = yf(k(ki));
        daci(j,1:ni) = dac(k(ki));
        ti(j) = nanmedian(t(k(ki)));
    end
    
    la = lati(:);
    la(la == 999) = [];
    lo = loni(:);
    lo(lo == 999) = [];
    [la,iu] = unique(la);
    lo = lo(iu);
    h = NaN(ncycle,length(la));
    hf = NaN(ncycle,length(la));
    h_dac = NaN(ncycle,length(la));
    for j=1:ncycle
        [~,i1,i2] = intersect(la,lati(j,:)');
        h(j,i1) = yi(j,i2); 
        hf(j,i1) = yfi(j,i2); 
        h_dac(j,i1) = daci(j,i2);
    end
    
    clear data
    data.lon = lo;
    data.lat = la;
    data.time = ti;
    data.sla = h;
    data.sla_filtered = hf;
    data.dac = h_dac; % to revert dac: twle = sla + dac
    data.track = track_unique(i);
    
    % save to netcdf
    if strcmp(alti,'jason')
        fileName = ['j1j2j3_cmems_p',num2str(track_unique(i),'%04d'),'.nc'];
    elseif strcmp(alti,'s3a')
        fileName = ['s3a_cmems_p',num2str(track_unique(i),'%04d'),'.nc'];
    elseif strcmp(alti,'s3b')
        fileName = ['s3b_cmems_p',num2str(track_unique(i),'%04d'),'.nc'];
    end
    save2netcdf(data,fileName,pathout)
    
    % save to mat file
%     if strcmp(alti,'jason')
%         save([pathout,'j1j2j3_cmems_p',num2str(track_unique(i),'%04d'),'.mat'],'-struct','data','-mat')
%     elseif strcmp(alti,'s3a')
%         save([pathout,'s3a_cmems_p',num2str(track_unique(i),'%04d'),'.mat'],'-struct','data','-mat')
%     elseif strcmp(alti,'s3b')
%         save([pathout,'s3b_cmems_p',num2str(track_unique(i),'%04d'),'.mat'],'-struct','data','-mat')
%     end
end
