#import sys
#sys.path.append("/mnt/Windows/anaconda/lib/python2.7/site-packages/")
import scipy as sp
import numpy as np
import sys
import cPickle
from scipy.io import loadmat
from os import listdir
from readTgData import readTg
#from readAltiData import readAlti
from readAltiData_nc import readAlti #added 27 Feb 2018 (reads Alti from new colocated netcdf data)
from minDistance import minDistance
import datetime
from plot_rho_distCoast import plot_rho_distCoast
from plot_rms_distCoast import plot_rms_distCoast
from plot_series_maxCorr import plot_series_maxCorr
from plot_tg_passes import plot_passes
from seasonalCycle import seasonalCycle
from readTgDataMonthly import readTgMonthly
import time
import warnings


def tgAltiStats(tgName,distMax,dx,path2Alti,path2Tg,path2TgMonthly,path2Coast,pathOut):

    dcoast = np.arange(dx,100+dx,dx) # in km
    # read Tg data
    tgName = tgName.replace(" ","_")
    dataTg = readTg(path2Tg,tgName)
    #tgName = dataTg["tgName"]
    xTg = dataTg["lon"]
    yTg = dataTg["lat"]
    tTg = dataTg["dateEpoch"]
    hTg = dataTg["tidal_res"]
    tTg = tTg[~np.isnan(hTg)] # remove NaNs from TG data
    hTg = hTg[~np.isnan(hTg)]

    # select altimetry passes that have data within distMax km from the tide gauge
    listAlti = listdir(path2Alti) # list files in directory
    passes = [x[0:-3].split('p')[-1] for x in listAlti if 'j1j2' in x]
    maxd = (100 + distMax) - np.mod(distMax,100)
    dataAlti = []
    for p in passes:
	dati = readAlti(path2Alti,p)
	xa = dati["lon"]
	ya = dati["lat"]
	knan = np.any(~np.isnan(dati["ssha"]),0)
	dmin,kmin = minDistance(xTg,yTg,xa[knan],ya[knan],maxd) # ya is sorted
	if dmin[0]/1000.<=distMax:
	    dataAlti.append(dati)
	    
    # save coordinates for selected altimetry passes
    pass_xy = {}
    for dati in dataAlti:
        xi = dati["lon"]
        yi = dati["lat"]
        xy = np.array([xi,yi]).T
        pass_xy[dati["passNumber"]] = xy
    
    with open(pathOut + 'passes_' + tgName, 'wb') as fp:
	cPickle.dump(pass_xy,fp)

    # plot map with passes
    plot_passes(tgName,xTg,yTg)
	
    # compute distance from coast and TG, and store it
    gshhs = loadmat(path2Coast + "gshhs_full.mat",struct_as_record=False)
    gshhs = gshhs["gshhs_f"][0,0]
    xc = gshhs.lon[0,:]
    yc = gshhs.lat[0,:]
    knan = ~np.isnan(xc)
    xc = xc[knan]
    yc = yc[knan]
    kregion = (yc>-50) & (yc<0) & (xc>-10) & (xc<70)
    xc = xc[kregion]
    yc = yc[kregion]
    ksort = np.argsort(yc)
    yc = yc[ksort]
    xc = xc[ksort]
    for z in dataAlti:
	xa = z["lon"]
	ya = z["lat"]
	disti,temp = minDistance(xa,ya,xc,yc,maxd)
	z["dist2Coast"] = disti/1000. # km
	disti,temp = minDistance(xa,ya,xTg,yTg,maxd)
	z["dist2Tg"] = disti/1000. # km
	
    # compute comparison pairs, correlation, and RMS difference
    tgAltiStats = []
    for dc in dcoast:
	kdc = [np.nonzero((x["dist2Coast"] >= dc -dx) & (x["dist2Coast"] < dc) &
	      (np.any(~np.isnan(x["ssha"]) & (x["dist2Tg"] < distMax),0)))[0] for x in dataAlti]

	statPass = {}
	for i,z in enumerate(dataAlti): # loop over passes
	    kdci = kdc[i]
	    if kdci.shape[0] != 0:
		zi = z.copy()
		kdci = kdc[i]
		ha = zi["ssha"][:,kdci]
		dac = zi["dac"][:,kdci]
		distc = zi["dist2Coast"][kdci]
		distTg = zi["dist2Tg"][kdci]
		xa = zi["lon"][kdci]
		ya = zi["lat"][kdci]
		ta = zi["time"][:,kdci]

		# remove outliers
		with np.errstate(invalid='ignore'):
		    ha.unshare_mask()
		    kout = np.nonzero((np.absolute(ha - np.nanmean(ha,0)) > 2.)) # remove points ssha > 2 m
		    ha[kout] = np.NaN
		    kout = np.nonzero((np.absolute(ha - np.nanmean(ha,0)) > 3.*np.nanstd(ha,0))) # remove ssha > 3std
		    ha[kout] = np.NaN

		# compute average over selected points along pass
		with warnings.catch_warnings():
		    warnings.simplefilter("ignore", category=RuntimeWarning)
		    ha = np.nanmean(ha,1)
		    dac = np.nanmean(dac,1)
		    distc = np.nanmean(distc)
		    distTg = np.nanmean(distTg)
		    ta = np.nanmean(ta,1)

		# store full altimetry time series
		haFull = ha.copy()
		dacFull = dac.copy()
		taFull = ta.copy() 
		knan = np.where(~np.isnan(taFull))[0]
		haFull = haFull[knan]
                dacFull = dacFull[knan]
                taFull = taFull[knan]

		# remove records for which the temporal separation with TG is more than 2 h
		for ii in range(ta.shape[0]):
		    dtmin = np.abs(ta[ii] - tTg)
		    dtmin = np.amin(dtmin)/3600.
		    if dtmin > 2:
			ha[ii] = np.NaN
		knan = ~np.isnan(ha)
		ha = ha[knan]
		dac = dac[knan]
		ta = ta[knan]

		# interpolate TG data to time of pass
		hTgi = np.interp(ta,tTg,hTg)
		
	        if ha.shape[0] == 0 or hTgi.shape[0] == 0:
	            print "No data meet the imposed criteria. Exiting..."
	            #sys.exit(1)
	            return

		#compute correlation and RMS difference
		rho = np.corrcoef(ha + dac,hTgi)[0,1]
		rms = np.std((ha + dac) - hTgi)

		# store results in a dictionary
		statResult = {"ssha":ha,"dac":dac,"tgRes":hTgi,"time":ta,"dist2Coast":distc,"dist2Tg":distTg,"corr":rho,
			       "rms":rms,"passNum":z["passNumber"],"sshaFull":haFull,"dacFull":dacFull,"timeFull":taFull}

		# store in list by pass
		statPass[z["passNumber"]] = statResult
	
	# store results in list by distance from coast
	tgAltiStats.append(statPass)

    # plot correlation and RMS as a function of distance from coast for all passes
    ndist = dcoast.shape[0]
    isdata = [x for x in tgAltiStats if x]
    if not isdata:
	print "No data meet the imposed criteria. Exiting..."
	#sys.exit(1)
	return
    stats = {}
    for z in dataAlti:
	stats[z["passNumber"]] = {}
	stats[z["passNumber"]]["corr"] = np.zeros((ndist,))*np.NaN
	stats[z["passNumber"]]["rms"] = np.zeros((ndist,))*np.NaN
	stats[z["passNumber"]]["dist2Coast"] = np.zeros((ndist,))*np.NaN
	stats[z["passNumber"]]["dist2Tg"] = np.zeros((ndist,))*np.NaN

    for i,z in enumerate(tgAltiStats):
	for p in z.keys(): #loop over passes
	    stats[p]["corr"][i] = z[p]["corr"]
	    stats[p]["rms"][i] = z[p]["rms"]
	    stats[p]["dist2Coast"][i] = z[p]["dist2Coast"]
	    stats[p]["dist2Tg"][i] = z[p]["dist2Tg"]

    passes = stats.keys()
    for p in passes:
	z = stats[p]
	if any(~np.isnan(z["corr"])):
	    plot_rho_distCoast(z["corr"],dcoast,z["dist2Tg"],tgName,p)
	    plot_rms_distCoast(z["rms"]*100.,dcoast,z["dist2Tg"],tgName,p)


    # find distance and pass giving max correlation
    kmax = np.zeros((len(passes),))
    rmax = np.zeros((len(passes),))
    for i,p in enumerate(passes):
	r = stats[p]["corr"]
	if np.any(~np.isnan(r)):
	    kmax[i] = np.nanargmax(r)
	    rmax[i] = r[int(kmax[i])]
    ki = np.nanargmax(rmax)
    pmax = passes[ki]
    kmax = int(kmax[ki])

    # extract time series with max correlation
    series = tgAltiStats[kmax][pmax]

    # plot series giving max correlation
    y1 = series["ssha"] + series["dac"]
    y2 = series["tgRes"]
    knan = np.where(~np.isnan(y1))[0]
    if knan.shape[0]<30:
        print "No enough data to compute robust statistics. Exiting..."
	#sys.exit(1)
	return
    y1 = y1 - np.nanmean(y1)
    y2 = y2 - np.nanmean(y2)
    y1 = y1*100.
    y2 = y2*100.
    t = series["time"]
    td = [datetime.datetime.utcfromtimestamp(x) for x in t.tolist()]
    t0 = [datetime.datetime(x.year,1,1,0,0) for x in td]
    tf = [datetime.datetime(x.year+1,1,1,0,0,0) for x in td]
    tdec = [x.year + (x - y).days/float((z - y).days) for x,y,z in zip(td,t0,tf)]
    plot_series_maxCorr(tdec,y1,y2,tgName,pmax)

    # compute seasonal cycle
    # 1. Altimetry 10 days
    y = series["ssha"] + series["dac"]
    t = series["time"]
    y = y*100.
    seasa = seasonalCycle(y,t)

    # 2. Altimetry 'monthly'
    y = series["sshaFull"] + series["dacFull"]
    t = series["timeFull"]
    td = [datetime.datetime.utcfromtimestamp(x) for x in t.tolist()]
    y = y*100.
    nmonths = int((td[-1].year - td[0].year + 1)*12.)
    Y = [x.year for x in td]
    M = [x.month for x in td]
    Y = np.asarray(Y)
    M = np.asarray(M)
    ym = np.zeros((nmonths,))*np.NaN
    tm = np.zeros((nmonths,))*np.NaN
    ct = -1
    for yr in np.arange(Y[0],Y[-1]+1,1):
	for m in np.arange(1,13,1):
	    ct = ct+1
	    k = np.nonzero((Y==yr) & (M==m))[0]
	    if k.shape[0] == 3:
                with warnings.catch_warnings():
		    warnings.simplefilter("ignore", category=RuntimeWarning)
		    ym[ct] = np.nanmean(y[k])
		    tm[ct] = np.nanmean(t[k])
    knan = np.where(~np.isnan(ym) & ~np.isnan(tm))[0]
    knan = range(knan[0],knan[-1]+1,1)
    ym = ym[knan]
    tm = tm[knan]
    seasam = seasonalCycle(ym,tm)
    series["sshaFullMonthly"] = ym
    series["timeFullMonthly"] = tm

    # 3. Tide gauge with altimetry sampling
    y = series["tgRes"]
    t = series["time"]
    y = y*100.
    seasg = seasonalCycle(y,t)

    # 4. Tide gauge monthly 
    tgMonthly = readTgMonthly(path2TgMonthly,tgName)
    ym = tgMonthly["sea_level"]
    tm = tgMonthly["dateEpoch"]
    ym = (ym - np.mean(ym[ym==ym]))*100
    if any(~np.isnan(ym)):
        seasgm = seasonalCycle(ym,tm)
    else:
        seasgm = {"amp_an":np.NaN,"amp_an_se":np.NaN,"phase_an":np.NaN,"phase_an_se":np.NaN,"amp_semian":np.NaN,
               "amp_semian_se":np.NaN,"phase_semian":np.NaN,"phase_semian_se":np.NaN,"residual":np.NaN}
    series["tgMonthly"] = ym
    series["timeTgMonthly"] = tm


    # save results to file for later reading
    series["lonTg"] = xTg
    series["latTg"] = yTg
    series["seasonal_alti"] = seasa
    series["seasonal_alti_monthly"] = seasam
    series["seasonal_tg_altiSampling"] = seasg
    series["seasonal_tg_monthly"] = seasgm
    with open(pathOut + 'data_' + tgName, 'wb') as fp:
	cPickle.dump(series,fp)

	    
	    




     
